"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useEndpointIntegrationWrapper = exports.PARAM_VALIDATION_ERROR_TO_MESSAGE = void 0;
var parameterValidation_1 = require("./parameterValidation");
var LoggerChalked_1 = require("../modules/LoggerChalked");
/**
 * Useful Error Messages for Validation Errors.
 * Can be used as an Error Message to send to the client
 */
exports.PARAM_VALIDATION_ERROR_TO_MESSAGE = {
    MISSING_ID: 'Please specify a locale in the body through: e.g. "locale": "de_DE" ',
    MISSING_LOCALE: 'Please Provide an id in the body',
    MISSING_PARAM_OBJECT: "Please provide required params in the body. required: 'id', 'locale'",
    MISSING_FILTERS: 'Please provide non-empty filters array',
};
/**
 * Provides Wrapper Functions fot the given api.
 * @param api FSXAApi to wrap
 * @param loggerName Creates a logger with the given name
 * @returns
 */
var useEndpointIntegrationWrapper = function (api, loggerName) {
    if (loggerName === void 0) { loggerName = 'Endpoint-Utils'; }
    var logger = new LoggerChalked_1.LoggerChalked(api.logLevel, loggerName);
    /**
     * Fetches an Element by Id via {@link FSXARemoteApi}
     * @param params
     * @param logContext
     * @returns A ResultWrapper, that either indicates success and contains the data, or indicates an Error and a Reason
     */
    var fetchElementWrapper = function (params, logContext) {
        if (logContext === void 0) { logContext = '[fetchElement]'; }
        return __awaiter(void 0, void 0, void 0, function () {
            var validationResult, response, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        logger.debug(logContext, 'Called');
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        validationResult = (0, parameterValidation_1.validateParamsFetchElement)(params);
                        if (!validationResult.valid) {
                            return [2 /*return*/, handleValidationError(validationResult, params, logContext)];
                        }
                        logger.debug(logContext, 'Params valid, fetching from Api');
                        return [4 /*yield*/, api.fetchElement({
                                id: params.id,
                                locale: params.locale,
                                additionalParams: params.additionalParams,
                                remoteProject: params.remote,
                                filterContext: params.filterContext,
                                normalized: params.normalized,
                            })];
                    case 2:
                        response = _a.sent();
                        logger.debug(logContext, 'fetch successful');
                        return [2 /*return*/, {
                                success: true,
                                data: response,
                            }];
                    case 3:
                        error_1 = _a.sent();
                        return [2 /*return*/, handleRuntimeError(error_1, logContext)];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Fetches Navigation via {@link FSXARemoteApi}
     * @param params
     * @param logContext
     * @returns A ResultWrapper, that either indicates success and contains the data, or indicates an Error and a Reason
     */
    var fetchNavigationWrapper = function (params, logContext) {
        if (logContext === void 0) { logContext = '[fetchNavigation]'; }
        return __awaiter(void 0, void 0, void 0, function () {
            var validationResult, response, error_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        logger.debug(logContext, 'Called');
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        validationResult = (0, parameterValidation_1.validateParamsFetchNavigation)(params);
                        if (!validationResult.valid) {
                            return [2 /*return*/, handleValidationError(validationResult, params, logContext)];
                        }
                        logger.debug(logContext, 'Params valid, fetching from Api');
                        return [4 /*yield*/, api.fetchNavigation({
                                initialPath: params.initialPath || '/',
                                locale: params.locale,
                                filterContext: params.filterContext,
                            })];
                    case 2:
                        response = _a.sent();
                        logger.debug(logContext, 'fetch successful');
                        return [2 /*return*/, {
                                success: true,
                                data: response,
                            }];
                    case 3:
                        error_2 = _a.sent();
                        return [2 /*return*/, handleRuntimeError(error_2, logContext)];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Fetches Elements by Filter via {@link FSXARemoteApi}
     * @param params
     * @param logContext
     * @returns A ResultWrapper, that either indicates success and contains the data, or indicates an Error and a Reason
     */
    var fetchByFilterWrapper = function (params, logContext) {
        if (logContext === void 0) { logContext = '[fetchByFilter]'; }
        return __awaiter(void 0, void 0, void 0, function () {
            var validationResult, response, error_3;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        logger.debug(logContext, 'Called');
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        validationResult = (0, parameterValidation_1.validateParamsFetchByFilter)(params);
                        if (!validationResult.valid) {
                            return [2 /*return*/, handleValidationError(validationResult, params, logContext)];
                        }
                        logger.debug(logContext, 'Params valid, fetching from Api');
                        return [4 /*yield*/, api.fetchByFilter({
                                filters: params.filters || [],
                                locale: params.locale,
                                page: params.page ? params.page : undefined,
                                pagesize: params.pagesize ? params.pagesize : undefined,
                                sort: params.sort ? params.sort : [],
                                additionalParams: params.additionalParams || {},
                                remoteProject: params.remote ? params.remote : undefined,
                                filterContext: params.filterContext,
                                normalized: params === null || params === void 0 ? void 0 : params.normalized,
                            })];
                    case 2:
                        response = _a.sent();
                        logger.debug(logContext, 'fetch successful');
                        return [2 /*return*/, {
                                success: true,
                                data: response,
                            }];
                    case 3:
                        error_3 = _a.sent();
                        return [2 /*return*/, handleRuntimeError(error_3, logContext)];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Fetches ProjectProperties via {@link FSXARemoteApi}
     * @param params
     * @param logContext
     * @returns A ResultWrapper, that either indicates success and contains the data, or indicates an Error and a Reason
     */
    var fetchProjectPropertiesWrapper = function (params, logContext) {
        if (logContext === void 0) { logContext = '[fetchProjectProperties]'; }
        return __awaiter(void 0, void 0, void 0, function () {
            var validationResult, response, error_4;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        logger.debug(logContext, 'Called');
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        validationResult = (0, parameterValidation_1.validateParamsFetchProjectProperties)(params);
                        if (!validationResult.valid) {
                            return [2 /*return*/, handleValidationError(validationResult, params, logContext)];
                        }
                        logger.debug(logContext, 'Params valid, fetching from Api');
                        return [4 /*yield*/, api.fetchProjectProperties({
                                locale: params.locale,
                                additionalParams: params.additionalParams,
                                resolve: params.resolver,
                                filterContext: params.filterContext,
                                normalized: params.normalized,
                            })];
                    case 2:
                        response = _a.sent();
                        logger.debug(logContext, 'fetch successful');
                        return [2 /*return*/, {
                                success: true,
                                data: response,
                            }];
                    case 3:
                        error_4 = _a.sent();
                        return [2 /*return*/, handleRuntimeError(error_4, logContext)];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    var handleValidationError = function (validationResult, params, logContext) {
        logger.error(logContext, validationResult.reason, params);
        return {
            success: false,
            error: {
                statusCode: validationResult.statusCode,
                message: exports.PARAM_VALIDATION_ERROR_TO_MESSAGE[validationResult.reason],
            },
            originalError: validationResult,
        };
    };
    function assertIsError(error) {
        // rethrow unknown error if its not an instance of error, since we do not know how to handle that
        if (!(error instanceof Error)) {
            throw error;
        }
    }
    var handleRuntimeError = function (error, logContext) {
        assertIsError(error);
        logger.error(logContext, error);
        var errorObj = {
            statusCode: error.statusCode || 500,
            message: error.message,
        };
        return {
            success: false,
            error: errorObj,
            originalError: error,
        };
    };
    return {
        fetchElementWrapper: fetchElementWrapper,
        fetchNavigationWrapper: fetchNavigationWrapper,
        fetchByFilterWrapper: fetchByFilterWrapper,
        fetchProjectPropertiesWrapper: fetchProjectPropertiesWrapper,
    };
};
exports.useEndpointIntegrationWrapper = useEndpointIntegrationWrapper;
//# sourceMappingURL=endpointIntegrationWrapper.js.map