"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.eventStreamHandler = void 0;
var better_sse_1 = require("better-sse");
var reconnecting_websocket_1 = __importDefault(require("reconnecting-websocket"));
var ws_1 = __importDefault(require("ws"));
var LoggerChalked_1 = require("./LoggerChalked");
/**
 * The CaaSEventStream is used for a unique (per project) CaaS change event websocket and delegates (channel broadcast)
 * any incoming events to any registered event-stream (session)
 */
var CaaSEventStream = /** @class */ (function () {
    function CaaSEventStream(logger, createSocketUrl, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        this.logger = logger;
        this.createSocketUrl = createSocketUrl;
        this.channel = (0, better_sse_1.createChannel)();
        this.channel.on('session-registered', function () { return _this.onCheckState(); });
        this.channel.on('session-deregistered', function () { return _this.onCheckState(); });
        this.socket = new reconnecting_websocket_1.default(this.createSocketUrl.bind(this), [], Object.assign({ WebSocket: ws_1.default, startClosed: true }, options));
        this.socket.onmessage = this.onSocketMessage.bind(this);
        this.socket.onerror = function (event) {
            _this.logger.warn('onSocketError', 'error', event.message);
        };
        this.logger.info('initilized');
    }
    // on any session (de)registration, close or (re)open then websocket
    CaaSEventStream.prototype.onCheckState = function () {
        var hasSessions = this.channel.activeSessions.length > 0;
        var isSocketOpenOrConnecting = this.socket.readyState === reconnecting_websocket_1.default.OPEN ||
            this.socket.readyState === reconnecting_websocket_1.default.CONNECTING;
        this.logger.info('onCheckState', 'activeSessions', this.channel.activeSessions.length);
        if (hasSessions && !isSocketOpenOrConnecting) {
            this.socket.reconnect();
        }
        else if (!hasSessions && isSocketOpenOrConnecting) {
            this.socket.close();
        }
    };
    // receive websocket message and broadcast to all open event-streams
    CaaSEventStream.prototype.onSocketMessage = function (event) {
        var jsonStringMessage = event.data.toString('utf-8');
        this.logger.info('onSocketMessage', 'broadcast', jsonStringMessage);
        this.channel.broadcast(jsonStringMessage);
    };
    // add a new event-stream (session)
    CaaSEventStream.prototype.addSession = function (req, res) {
        var _this = this;
        var serializer = function (data) {
            return typeof data === 'string' ? data : JSON.stringify(data);
        };
        (0, better_sse_1.createSession)(req, res, { serializer: serializer }).then(function (session) {
            return _this.channel.register(session);
        });
    };
    return CaaSEventStream;
}());
var streams = {};
var eventStreamHandler = function (api) {
    return function (req, res) { return __awaiter(void 0, void 0, void 0, function () {
        var logger, remoteProject, caasUrl, createSocketUrl;
        return __generator(this, function (_a) {
            logger = new LoggerChalked_1.LoggerChalked(api.logLevel, 'CaaSEventStream');
            logger.info('requesting route: ', req.url, req.query);
            if ('remoteProject' in req.query) {
                remoteProject = "" + req.query.remoteProject;
            }
            caasUrl = api.buildCaaSUrl({ remoteProject: remoteProject }).split('?')[0];
            // ensure to have only one socket connection per caasUrl
            if (!(caasUrl in streams)) {
                createSocketUrl = function () { return __awaiter(void 0, void 0, void 0, function () {
                    var token, socketUrl;
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4 /*yield*/, api.fetchSecureToken()];
                            case 1:
                                token = _a.sent();
                                socketUrl = caasUrl.replace(/^http/, 'ws') + "/_streams/crud?securetoken=" + token;
                                logger.info('createSocketUrl', 'socketUrl', socketUrl);
                                return [2 /*return*/, socketUrl];
                        }
                    });
                }); };
                streams[caasUrl] = new CaaSEventStream(logger, createSocketUrl);
            }
            // add this connection to a pool (channel) of listeners for the websocket events
            streams[caasUrl].addSession(req, res);
            return [2 /*return*/];
        });
    }); };
};
exports.eventStreamHandler = eventStreamHandler;
//# sourceMappingURL=CaaSEventStream.js.map