"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CaaSMapper = exports.DEFAULT_MAX_REFERENCE_DEPTH = exports.CaaSMapperErrors = void 0;
var _1 = require("./");
var date_fns_1 = require("date-fns");
var lodash_1 = require("lodash");
var XMLParser_1 = __importDefault(require("./XMLParser"));
var Logger_1 = require("./Logger");
var enums_1 = require("../enums");
var MappingUtils_1 = require("./MappingUtils");
var CaaSMapperErrors;
(function (CaaSMapperErrors) {
    CaaSMapperErrors["UNKNOWN_BODY_CONTENT"] = "Unknown BodyContent could not be mapped.";
    CaaSMapperErrors["UNKNOWN_FSTYPE"] = "Unknown fsType could not be mapped";
})(CaaSMapperErrors = exports.CaaSMapperErrors || (exports.CaaSMapperErrors = {}));
var REFERENCED_ITEMS_CHUNK_SIZE = 30;
exports.DEFAULT_MAX_REFERENCE_DEPTH = 10;
var CaaSMapper = /** @class */ (function () {
    function CaaSMapper(api, locale, utils, logger) {
        var _this = this;
        var _a, _b;
        this.resolvedReferences = {};
        // stores references to items of current Project
        this._referencedItems = {};
        // stores References to remote Items
        this._remoteReferences = {};
        // stores items that are being or have been processed and should not be fetched
        // Again. They cannot be added to resolvedReferences yet, since the fetch call
        // just started, when we need to keep track of them.
        this._processedItems = {};
        this.api = api;
        this.locale = locale;
        this.customMapper = utils.customMapper;
        this.xmlParser = new XMLParser_1.default(logger);
        Object.values(this.api.remotes || {}).forEach(function (entry) { return (_this._remoteReferences[entry.id] = {}); });
        this.logger = logger;
        this.referenceDepth = (_a = utils.referenceDepth) !== null && _a !== void 0 ? _a : 0;
        this.maxReferenceDepth =
            (_b = utils.maxReferenceDepth) !== null && _b !== void 0 ? _b : exports.DEFAULT_MAX_REFERENCE_DEPTH;
        this.logger.debug('Created new CaaSMapper');
    }
    CaaSMapper.prototype.addToResolvedReferences = function (item, remoteProjectId) {
        // Page has pageId as id instead of PageRef Id. --> use refId instead for mapped Pages
        var id = (0, MappingUtils_1.getItemId)(item, remoteProjectId);
        if (id) {
            this.logger.debug('Add to resolvedReferences', id);
            this.resolvedReferences[id] = item;
        }
        else {
            this.logger.warn('No id for item', item);
        }
    };
    /**
     * unifies the two different id formats {id}.{locale} and {id} to {id}.{locale}
     * if remoteProjectConfiguration is passed, prefix with project id to avoid uuid clashes
     *
     * @param id uuid, may be of form {id}.{locale} or {id}
     * @param remoteProjectConfiguration remoteProjectConfig
     * @returns uuid of form {id}.{locale} or {remoteProjectId}#{id}.{locale}
     */
    CaaSMapper.prototype.unifyId = function (id, remoteProjectConfiguration) {
        var indexOfSeparator = id.indexOf('.');
        var remoteLocale = remoteProjectConfiguration === null || remoteProjectConfiguration === void 0 ? void 0 : remoteProjectConfiguration.locale;
        var idWithLocale;
        if (indexOfSeparator > 0) {
            // id has form {id}.{locale}. Override Locale if set
            idWithLocale = remoteLocale
                ? id.substring(0, indexOfSeparator) + "." + remoteLocale
                : id;
        }
        else {
            // id has form {id}. transform to {id}.{locale}
            idWithLocale = id + "." + (remoteLocale || this.locale);
        }
        return remoteProjectConfiguration
            ? remoteProjectConfiguration.id + "#" + idWithLocale
            : idWithLocale;
    };
    /**
     * registers a referenced Item to be fetched later. If a remoteProjectId is passed,
     * the item will be fetched from the remote Project.
     * @param identifier item identifier
     * @param path after fetch, items are inserted at all registered paths
     * @param remoteProjectId optional. If passed, the item will be fetched from the specified project
     * @returns placeholder string
     */
    CaaSMapper.prototype.registerReferencedItem = function (identifier, path, remoteProjectId, imageMapResolution) {
        var remoteData = this.getRemoteConfigForProject(remoteProjectId);
        var remoteProjectKey = remoteData === null || remoteData === void 0 ? void 0 : remoteData.id;
        if (remoteProjectId && !remoteProjectKey) {
            this.logger.warn("Item with identifier '" + identifier + "' was tried to register from remoteProject '" + remoteProjectId + "' but no remote key was found in the config.");
        }
        var unifiedId = this.unifyId(identifier, remoteData);
        // Preflight check to avoid costly operations in non debug mode
        this.logger.logLevel === Logger_1.LogLevel.DEBUG &&
            this.logger.debug('Registering Referenced Item ', {
                remoteProjectKey: remoteProjectKey,
                identifier: identifier,
                path: path.join('/'),
            });
        if (remoteProjectKey) {
            this._remoteReferences[remoteProjectKey][unifiedId] = __spreadArray(__spreadArray([], (this._remoteReferences[remoteProjectKey][unifiedId] || []), true), [
                path,
            ], false);
            return imageMapResolution
                ? "IMAGEMAP___" + imageMapResolution + "___" + unifiedId
                : "[REFERENCED-REMOTE-ITEM-" + unifiedId + "]";
        }
        this._referencedItems[unifiedId] = __spreadArray(__spreadArray([], (this._referencedItems[unifiedId] || []), true), [
            path,
        ], false);
        return imageMapResolution
            ? "IMAGEMAP___" + imageMapResolution + "___" + unifiedId
            : "[REFERENCED-ITEM-" + unifiedId + "]";
    };
    CaaSMapper.prototype.buildPreviewId = function (identifier, remoteProjectLocale) {
        return [identifier, remoteProjectLocale || this.locale].join('.');
    };
    CaaSMapper.prototype.buildMediaUrl = function (url, rev) {
        if (rev && this.api.contentMode === enums_1.FSXAContentMode.PREVIEW) {
            url += (url.includes('?') ? '&' : '?') + "rev=" + rev;
        }
        return url;
    };
    CaaSMapper.prototype.mapDataEntry = function (entry, path, remoteProjectLocale, remoteProjectId) {
        return __awaiter(this, void 0, void 0, function () {
            var result, _a, comboboxOption, richTextElements, _b, simpleValue, radiobuttonOption, dateValue, link, _c, reference, option, permission;
            var _d;
            var _this = this;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        if (!this.customMapper) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.customMapper(entry, path, {
                                api: this.api,
                                xmlParser: this.xmlParser,
                                registerReferencedItem: this.registerReferencedItem.bind(this),
                                buildPreviewId: this.buildPreviewId.bind(this),
                                buildMediaUrl: this.buildMediaUrl.bind(this),
                                mapDataEntries: this.mapDataEntries.bind(this),
                            })];
                    case 1:
                        result = _e.sent();
                        if (typeof result !== 'undefined')
                            return [2 /*return*/, result];
                        _e.label = 2;
                    case 2:
                        _a = entry.fsType;
                        switch (_a) {
                            case 'CMS_INPUT_COMBOBOX': return [3 /*break*/, 3];
                            case 'CMS_INPUT_DOM': return [3 /*break*/, 4];
                            case 'CMS_INPUT_DOMTABLE': return [3 /*break*/, 4];
                            case 'CMS_INPUT_NUMBER': return [3 /*break*/, 8];
                            case 'CMS_INPUT_TEXT': return [3 /*break*/, 8];
                            case 'CMS_INPUT_TEXTAREA': return [3 /*break*/, 8];
                            case 'CMS_INPUT_RADIOBUTTON': return [3 /*break*/, 9];
                            case 'CMS_INPUT_DATE': return [3 /*break*/, 10];
                            case 'CMS_INPUT_LINK': return [3 /*break*/, 11];
                            case 'CMS_INPUT_LIST': return [3 /*break*/, 16];
                            case 'CMS_INPUT_CHECKBOX': return [3 /*break*/, 17];
                            case 'CMS_INPUT_IMAGEMAP': return [3 /*break*/, 18];
                            case 'FS_DATASET': return [3 /*break*/, 19];
                            case 'CMS_INPUT_TOGGLE': return [3 /*break*/, 20];
                            case 'FS_CATALOG': return [3 /*break*/, 21];
                            case 'FS_REFERENCE': return [3 /*break*/, 22];
                            case 'FS_INDEX': return [3 /*break*/, 23];
                            case 'Option': return [3 /*break*/, 24];
                            case 'CMS_INPUT_PERMISSION': return [3 /*break*/, 25];
                        }
                        return [3 /*break*/, 26];
                    case 3:
                        comboboxOption = entry.value
                            ? {
                                type: 'Option',
                                key: entry.value.identifier,
                                value: entry.value.label,
                            }
                            : null;
                        return [2 /*return*/, comboboxOption];
                    case 4:
                        if (!entry.value) return [3 /*break*/, 6];
                        return [4 /*yield*/, this.xmlParser.parse(entry.value)];
                    case 5:
                        _b = _e.sent();
                        return [3 /*break*/, 7];
                    case 6:
                        _b = [];
                        _e.label = 7;
                    case 7:
                        richTextElements = _b;
                        return [2 /*return*/, this.mapLinksInRichTextElements(richTextElements, path, remoteProjectLocale, remoteProjectId)];
                    case 8:
                        simpleValue = entry.value;
                        return [2 /*return*/, simpleValue];
                    case 9:
                        radiobuttonOption = entry.value
                            ? __assign({ type: 'Option', key: entry.value.identifier, value: entry.value.label }, entry.value) : null;
                        return [2 /*return*/, radiobuttonOption];
                    case 10:
                        dateValue = entry.value
                            ? (0, date_fns_1.parseISO)(entry.value)
                            : null;
                        return [2 /*return*/, dateValue];
                    case 11:
                        if (!entry.value) return [3 /*break*/, 14];
                        _d = {
                            type: 'Link',
                            template: entry.value.template.uid
                        };
                        return [4 /*yield*/, this.mapDataEntries(entry.value.formData, __spreadArray(__spreadArray([], path, true), ['data'], false), remoteProjectLocale, remoteProjectId)];
                    case 12:
                        _d.data = _e.sent();
                        return [4 /*yield*/, this.mapDataEntries(entry.value.metaFormData, __spreadArray(__spreadArray([], path, true), ['meta'], false), remoteProjectLocale, remoteProjectId)];
                    case 13:
                        _c = (_d.meta = _e.sent(),
                            _d);
                        return [3 /*break*/, 15];
                    case 14:
                        _c = null;
                        _e.label = 15;
                    case 15:
                        link = _c;
                        return [2 /*return*/, link];
                    case 16:
                        if (!entry.value)
                            return [2 /*return*/, []];
                        return [2 /*return*/, Promise.all(entry.value.map(function (childEntry, index) {
                                return _this.mapDataEntry(childEntry, __spreadArray(__spreadArray([], path, true), [index], false), remoteProjectLocale, remoteProjectId);
                            }))];
                    case 17:
                        if (!entry.value)
                            return [2 /*return*/, []];
                        return [2 /*return*/, Promise.all(entry.value.map(function (childEntry, index) {
                                return _this.mapDataEntry(childEntry, __spreadArray(__spreadArray([], path, true), [index], false), remoteProjectLocale, remoteProjectId);
                            }))];
                    case 18:
                        if (!entry || !entry.value) {
                            return [2 /*return*/, null];
                        }
                        return [2 /*return*/, this.mapImageMap(entry, path, remoteProjectLocale, remoteProjectId)];
                    case 19:
                        if (!entry.value)
                            return [2 /*return*/, null];
                        if (Array.isArray(entry.value)) {
                            return [2 /*return*/, Promise.all(entry.value.map(function (childEntry, index) {
                                    return _this.mapDataEntry(childEntry, __spreadArray(__spreadArray([], path, true), [index], false), remoteProjectLocale, remoteProjectId);
                                }))];
                        }
                        else if (entry.value.fsType === 'DatasetReference') {
                            return [2 /*return*/, this.registerReferencedItem(entry.value.target.identifier, path, remoteProjectId)];
                        }
                        return [2 /*return*/, null];
                    case 20: return [2 /*return*/, entry.value || false];
                    case 21: return [2 /*return*/, Promise.all((entry.value || []).map(function (card, index) { return __awaiter(_this, void 0, void 0, function () {
                            var _a;
                            var _b;
                            return __generator(this, function (_c) {
                                switch (_c.label) {
                                    case 0:
                                        _a = card.template.fsType;
                                        switch (_a) {
                                            case 'SectionTemplate': return [3 /*break*/, 1];
                                            case 'LinkTemplate': return [3 /*break*/, 1];
                                            case 'PageTemplate': return [3 /*break*/, 2];
                                        }
                                        return [3 /*break*/, 4];
                                    case 1: return [2 /*return*/, this.mapSection(__assign(__assign({}, card), { fsType: 'Section', name: card.template.name, displayName: card.template.displayName }), __spreadArray(__spreadArray([], path, true), [index], false), remoteProjectLocale, remoteProjectId)];
                                    case 2:
                                        _b = {
                                            id: card.identifier,
                                            previewId: this.buildPreviewId(card.identifier, remoteProjectLocale),
                                            template: card.template.uid
                                        };
                                        return [4 /*yield*/, this.mapDataEntries(card.formData, __spreadArray(__spreadArray([], path, true), [index, 'data'], false), remoteProjectLocale, remoteProjectId)];
                                    case 3: return [2 /*return*/, (_b.data = _c.sent(),
                                            _b)];
                                    case 4: return [2 /*return*/, card];
                                }
                            });
                        }); }))];
                    case 22:
                        if (!entry.value)
                            return [2 /*return*/, null];
                        if (entry.value.fsType === 'Media') {
                            return [2 /*return*/, this.registerReferencedItem(entry.value.identifier, path, entry.value.remoteProject || remoteProjectId)];
                        }
                        else if (['PageRef', 'GCAPage'].includes(entry.value.fsType)) {
                            reference = {
                                type: 'Reference',
                                referenceId: entry.value.identifier,
                                referenceRemoteProject: entry.value.remoteProject || remoteProjectId,
                                referenceType: entry.value.fsType,
                            };
                            Object.keys(entry.value).includes('section') &&
                                Object.assign(reference, { section: entry.value.section });
                            return [2 /*return*/, reference];
                        }
                        return [2 /*return*/, entry];
                    case 23:
                        if (entry.dapType === 'DatasetDataAccessPlugin') {
                            return [2 /*return*/, entry.value
                                    .map(function (record, index) {
                                    var _a, _b;
                                    var identifier = (_b = (_a = record === null || record === void 0 ? void 0 : record.value) === null || _a === void 0 ? void 0 : _a.target) === null || _b === void 0 ? void 0 : _b.identifier;
                                    if (!identifier)
                                        return null;
                                    return _this.registerReferencedItem(identifier, __spreadArray(__spreadArray([], path, true), [index], false), remoteProjectId);
                                })
                                    .filter(Boolean)];
                        }
                        return [2 /*return*/, entry];
                    case 24:
                        option = {
                            type: 'Option',
                            key: entry.identifier,
                            value: entry.label,
                        };
                        return [2 /*return*/, option];
                    case 25:
                        permission = {
                            type: 'Permission',
                            fsType: entry.fsType,
                            name: entry.name,
                            value: entry.value.map(function (activity) {
                                return {
                                    allowed: activity.allowed.map(function (group) {
                                        return _this._mapPermissionGroup(group);
                                    }),
                                    forbidden: activity.forbidden.map(function (group) {
                                        return _this._mapPermissionGroup(group);
                                    }),
                                };
                            }),
                        };
                        return [2 /*return*/, permission];
                    case 26: return [2 /*return*/, entry];
                }
            });
        });
    };
    CaaSMapper.prototype.mapLinksInRichTextElements = function (richTextElements, path, remoteProjectLocale, remoteProjectId) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, Promise.all(richTextElements.map(function (richTextElement, index) { return __awaiter(_this, void 0, void 0, function () {
                            var link, _a;
                            var _b;
                            return __generator(this, function (_c) {
                                switch (_c.label) {
                                    case 0:
                                        if (!(richTextElement.type === 'link')) return [3 /*break*/, 2];
                                        _b = {
                                            type: 'Link',
                                            template: richTextElement.data.type
                                        };
                                        return [4 /*yield*/, this.mapDataEntries(richTextElement.data.data, __spreadArray(__spreadArray([], path, true), [index, 'data', 'data'], false), remoteProjectLocale, remoteProjectId)];
                                    case 1:
                                        link = (_b.data = _c.sent(),
                                            _b.meta = {},
                                            _b);
                                        richTextElement.data = link;
                                        _c.label = 2;
                                    case 2:
                                        if (!Array.isArray(richTextElement.content)) return [3 /*break*/, 4];
                                        _a = richTextElement;
                                        return [4 /*yield*/, this.mapLinksInRichTextElements(richTextElement.content, __spreadArray(__spreadArray([], path, true), [index, 'content'], false), remoteProjectLocale, remoteProjectId)];
                                    case 3:
                                        _a.content = _c.sent();
                                        _c.label = 4;
                                    case 4: return [2 /*return*/];
                                }
                            });
                        }); }))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, richTextElements];
                }
            });
        });
    };
    CaaSMapper.prototype._mapPermissionGroup = function (group) {
        return {
            groupId: group.groupPath.split('/').pop(),
            groupName: group.groupName,
            groupPath: group.groupPath,
        };
    };
    CaaSMapper.prototype.mapDataEntries = function (entries, path, remoteProjectLocale, remoteProjectId) {
        return __awaiter(this, void 0, void 0, function () {
            var keys, mappedEntries;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        keys = Object.keys(entries || {});
                        return [4 /*yield*/, Promise.all(Object.keys(entries || {}).map(function (key) {
                                return _this.mapDataEntry(entries[key], __spreadArray(__spreadArray([], path, true), [key], false), remoteProjectLocale, remoteProjectId);
                            }))];
                    case 1:
                        mappedEntries = _a.sent();
                        return [2 /*return*/, keys.reduce(function (result, key, index) {
                                var _a;
                                return (__assign(__assign({}, result), (_a = {}, _a[key] = mappedEntries[index], _a)));
                            }, {})];
                }
            });
        });
    };
    CaaSMapper.prototype.mapSection = function (section, path, remoteProjectLocale, remoteProjectId) {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = { id: section.identifier, type: 'Section', name: section.name, displayName: section.displayName, sectionType: section.template.uid, previewId: this.buildPreviewId(section.identifier, remoteProjectLocale) };
                        return [4 /*yield*/, this.mapDataEntries(section.formData, __spreadArray(__spreadArray([], path, true), ['data'], false), remoteProjectLocale, remoteProjectId)];
                    case 1: return [2 /*return*/, __assign.apply(void 0, [__assign.apply(void 0, [(_a.data = _b.sent(), _a.displayed = section.displayed, _a), (section.lifespan && {
                                    lifespan: __assign({ start: new Date(section.lifespan.start) }, (section.lifespan.end && { end: new Date(section.lifespan.end) })),
                                })]), { children: [] }])];
                }
            });
        });
    };
    CaaSMapper.prototype.mapContent2Section = function (content2Section, remoteProjectLocale) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, {
                        id: content2Section.identifier,
                        previewId: this.buildPreviewId(content2Section.identifier, remoteProjectLocale),
                        name: content2Section.name,
                        displayName: content2Section.displayName,
                        type: 'Section',
                        data: {
                            entityType: content2Section.entityType,
                            filterParams: content2Section.filterParams,
                            maxPageCount: content2Section.maxPageCount,
                            ordering: content2Section.ordering,
                            query: content2Section.query,
                            recordCountPerPage: content2Section.recordCountPerPage,
                            schema: content2Section.schema,
                        },
                        sectionType: content2Section.template.uid,
                        children: [],
                    }];
            });
        });
    };
    CaaSMapper.prototype.mapBodyContent = function (content, path, remoteProjectLocale, remoteProjectId) {
        var _a;
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_b) {
                switch (content.fsType) {
                    case 'Content2Section':
                        return [2 /*return*/, this.mapContent2Section(content, remoteProjectLocale)];
                    case 'Section':
                    case 'SectionReference':
                    case 'GCASection':
                        return [2 /*return*/, this.mapSection(content, path, remoteProjectLocale, remoteProjectId)];
                    default:
                        throw new Error(CaaSMapperErrors.UNKNOWN_BODY_CONTENT +
                            (" fsType=[" + ((_a = content) === null || _a === void 0 ? void 0 : _a.fsType) + "]"));
                }
                return [2 /*return*/];
            });
        });
    };
    CaaSMapper.prototype.mapPageBody = function (body, path, remoteProjectLocale, remoteProjectId) {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = {
                            type: 'PageBody',
                            name: body.name,
                            previewId: this.buildPreviewId(body.identifier, remoteProjectLocale)
                        };
                        return [4 /*yield*/, Promise.all(body.children.map(function (child, index) {
                                return _this.mapBodyContent(child, __spreadArray(__spreadArray([], path, true), ['children', index], false), remoteProjectLocale, remoteProjectId);
                            }))];
                    case 1: return [2 /*return*/, (_a.children = _b.sent(),
                            _a)];
                }
            });
        });
    };
    CaaSMapper.prototype.mapPageRef = function (pageRef, path, remoteProjectLocale, remoteProjectId) {
        if (path === void 0) { path = []; }
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b;
            var _c, _d;
            var _this = this;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        _c = { type: 'Page', id: pageRef.page.identifier, refId: pageRef.identifier, previewId: this.buildPreviewId(pageRef.identifier, remoteProjectLocale), name: pageRef.page.name, layout: pageRef.page.template.uid };
                        return [4 /*yield*/, Promise.all(pageRef.page.children.map(function (child, index) {
                                return _this.mapPageBody(child, __spreadArray(__spreadArray([], path, true), ['children', index], false), remoteProjectLocale, remoteProjectId);
                            }))];
                    case 1:
                        _c.children = _e.sent();
                        return [4 /*yield*/, this.mapDataEntries(pageRef.page.formData, __spreadArray(__spreadArray([], path, true), ['data'], false), remoteProjectLocale, remoteProjectId)];
                    case 2:
                        _c.data = _e.sent();
                        return [4 /*yield*/, this.mapDataEntries(pageRef.page.metaFormData, __spreadArray(__spreadArray([], path, true), ['meta'], false), remoteProjectLocale, remoteProjectId)];
                    case 3:
                        _a = [(_c.meta = _e.sent(), _c)];
                        _b = pageRef.metaFormData;
                        if (!_b) return [3 /*break*/, 5];
                        _d = {};
                        return [4 /*yield*/, this.mapDataEntries(pageRef.metaFormData, __spreadArray(__spreadArray([], path, true), ['metaPageRef'], false), remoteProjectLocale, remoteProjectId)];
                    case 4:
                        _b = (_d.metaPageRef = _e.sent(),
                            _d);
                        _e.label = 5;
                    case 5: return [2 /*return*/, __assign.apply(void 0, [__assign.apply(void 0, _a.concat([(_b)])), { remoteProjectId: remoteProjectId, route: pageRef.route }])];
                }
            });
        });
    };
    CaaSMapper.prototype.mapProjectProperties = function (properties, path, remoteProjectLocale, remoteProjectId) {
        var _a;
        if (path === void 0) { path = []; }
        return __awaiter(this, void 0, void 0, function () {
            var _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _b = {
                            type: 'ProjectProperties'
                        };
                        return [4 /*yield*/, this.mapDataEntries(properties.formData, __spreadArray(__spreadArray([], path, true), ['data'], false), remoteProjectLocale, remoteProjectId)];
                    case 1:
                        _b.data = _c.sent(),
                            _b.layout = properties.template.uid;
                        return [4 /*yield*/, this.mapDataEntries(properties.metaFormData, __spreadArray(__spreadArray([], path, true), ['meta'], false), remoteProjectLocale, remoteProjectId)];
                    case 2: return [2 /*return*/, (_b.meta = _c.sent(),
                            _b.name = properties.name,
                            _b.previewId = this.buildPreviewId(properties.identifier, remoteProjectLocale),
                            _b.id = properties.identifier,
                            _b.masterLocale = (_a = properties.projectConfiguration) === null || _a === void 0 ? void 0 : _a.masterLocale,
                            _b.remoteProjectId = remoteProjectId,
                            _b)];
                }
            });
        });
    };
    CaaSMapper.prototype.mapImageMapArea = function (area, path, remoteProjectLocale, remoteProjectId) {
        return __awaiter(this, void 0, void 0, function () {
            var base, _a;
            var _b, _c;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        _b = {
                            areaType: area.areaType
                        };
                        _a = area.link;
                        if (!_a) return [3 /*break*/, 2];
                        _c = {
                            template: area.link.template.uid
                        };
                        return [4 /*yield*/, this.mapDataEntries(area.link.formData, __spreadArray(__spreadArray([], path, true), ['link', 'data'], false), remoteProjectLocale, remoteProjectId)];
                    case 1:
                        _a = (_c.data = _d.sent(),
                            _c);
                        _d.label = 2;
                    case 2:
                        base = (_b.link = _a,
                            _b);
                        switch (area.areaType) {
                            case enums_1.ImageMapAreaType.RECT:
                                return [2 /*return*/, __assign(__assign({}, base), { leftTop: area.leftTop, rightBottom: area.rightBottom })];
                            case enums_1.ImageMapAreaType.CIRCLE:
                                return [2 /*return*/, __assign(__assign({}, base), { center: area.center, radius: area.radius })];
                            case enums_1.ImageMapAreaType.POLY:
                                return [2 /*return*/, __assign(__assign({}, base), { points: area.points })];
                            default:
                                return [2 /*return*/, null];
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    CaaSMapper.prototype.mapImageMap = function (imageMap, path, remoteProjectLocale, remoteProjectId) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, media, areas, resolution, mappedAreas, image;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!imageMap.value) {
                            throw new Error('ImageMap value is null');
                        }
                        _a = imageMap.value, media = _a.media, areas = _a.areas, resolution = _a.resolution;
                        this.logger.debug('CaaSMapper.mapImageMap - imageMap', imageMap);
                        return [4 /*yield*/, Promise.all(areas.map(function (area, index) { return __awaiter(_this, void 0, void 0, function () {
                                return __generator(this, function (_a) {
                                    return [2 /*return*/, this.mapImageMapArea(area, __spreadArray(__spreadArray([], path, true), ['areas', index], false), remoteProjectLocale, remoteProjectId)];
                                });
                            }); }))];
                    case 1:
                        mappedAreas = _b.sent();
                        image = null;
                        if (media) {
                            image = this.registerReferencedItem(media.identifier, __spreadArray(__spreadArray([], path, true), ['media'], false), media.remoteProject, resolution.uid);
                        }
                        return [2 /*return*/, {
                                type: 'ImageMap',
                                areas: mappedAreas.filter(Boolean),
                                // image is just a "fake" image, it's a string that is used for an unresolved reference (see also TNG-1169)
                                media: image,
                            }];
                }
            });
        });
    };
    CaaSMapper.prototype.mapGCAPage = function (gcaPage, path, remoteProjectLocale, remoteProjectId) {
        if (path === void 0) { path = []; }
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = {
                            type: 'GCAPage',
                            id: gcaPage.identifier,
                            previewId: this.buildPreviewId(gcaPage.identifier, remoteProjectLocale),
                            name: gcaPage.name,
                            layout: gcaPage.template.uid
                        };
                        return [4 /*yield*/, Promise.all(gcaPage.children.map(function (child, index) {
                                return _this.mapPageBody(child, __spreadArray(__spreadArray([], path, true), ['children', index], false), remoteProjectLocale, remoteProjectId);
                            }))];
                    case 1:
                        _a.children = _b.sent();
                        return [4 /*yield*/, this.mapDataEntries(gcaPage.formData, __spreadArray(__spreadArray([], path, true), ['data'], false), remoteProjectLocale, remoteProjectId)];
                    case 2:
                        _a.data = _b.sent();
                        return [4 /*yield*/, this.mapDataEntries(gcaPage.metaFormData, __spreadArray(__spreadArray([], path, true), ['meta'], false), remoteProjectLocale, remoteProjectId)];
                    case 3: return [2 /*return*/, (_a.meta = _b.sent(),
                            _a.remoteProjectId = remoteProjectId,
                            _a)];
                }
            });
        });
    };
    CaaSMapper.prototype.mapDataset = function (dataset, path, remoteProjectLocale, remoteProjectId) {
        var _a;
        if (path === void 0) { path = []; }
        return __awaiter(this, void 0, void 0, function () {
            var _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _b = {
                            type: 'Dataset',
                            id: dataset.identifier,
                            previewId: this.buildPreviewId(dataset.identifier, remoteProjectLocale),
                            schema: dataset.schema,
                            entityType: dataset.entityType
                        };
                        return [4 /*yield*/, this.mapDataEntries(dataset.formData, __spreadArray(__spreadArray([], path, true), ['data'], false), remoteProjectLocale, remoteProjectId)];
                    case 1: return [2 /*return*/, (_b.data = _c.sent(),
                            _b.route = dataset.route,
                            _b.routes = dataset.routes,
                            _b.template = (_a = dataset.template) === null || _a === void 0 ? void 0 : _a.uid,
                            _b.children = [],
                            _b.remoteProjectId = remoteProjectId,
                            _b.locale = dataset.locale.language + '_' + dataset.locale.country,
                            _b)];
                }
            });
        });
    };
    CaaSMapper.prototype.mapMediaPicture = function (item, path, remoteProjectLocale, remoteProjectId) {
        var _a;
        if (path === void 0) { path = []; }
        return __awaiter(this, void 0, void 0, function () {
            var _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _b = {
                            type: 'Image',
                            id: item.identifier,
                            previewId: this.buildPreviewId(item.identifier, remoteProjectLocale)
                        };
                        return [4 /*yield*/, this.mapDataEntries(item.metaFormData, __spreadArray(__spreadArray([], path, true), ['meta'], false), remoteProjectLocale, remoteProjectId)];
                    case 1: return [2 /*return*/, (_b.meta = _c.sent(),
                            _b.description = item.description,
                            _b.resolutions = this.mapMediaPictureResolutionUrls(item.resolutionsMetaData, (_a = item.changeInfo) === null || _a === void 0 ? void 0 : _a.revision),
                            _b.remoteProjectId = remoteProjectId,
                            _b)];
                }
            });
        });
    };
    CaaSMapper.prototype.mapMediaPictureResolutionUrls = function (resolutions, rev) {
        for (var resolution in resolutions) {
            resolutions[resolution].url = this.buildMediaUrl(resolutions[resolution].url, rev);
        }
        return resolutions;
    };
    CaaSMapper.prototype.mapMediaFile = function (item, path, remoteProjectLocale, remoteProjectId) {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = {
                            type: 'File',
                            id: item.identifier,
                            previewId: this.buildPreviewId(item.identifier, remoteProjectLocale)
                        };
                        return [4 /*yield*/, this.mapDataEntries(item.metaFormData, __spreadArray(__spreadArray([], path, true), ['meta'], false), remoteProjectLocale, remoteProjectId)];
                    case 1: return [2 /*return*/, (_a.meta = _b.sent(),
                            _a.fileName = item.fileName,
                            _a.fileMetaData = item.fileMetaData,
                            _a.url = item.url,
                            _a.remoteProjectId = remoteProjectId,
                            _a)];
                }
            });
        });
    };
    CaaSMapper.prototype.mapMedia = function (item, path, remoteProjectLocale, remoteProjectId) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (item === null) {
                    return [2 /*return*/, null];
                }
                switch (item.mediaType) {
                    case 'PICTURE':
                        return [2 /*return*/, this.mapMediaPicture(item, path, remoteProjectLocale, remoteProjectId)];
                    case 'FILE':
                        return [2 /*return*/, this.mapMediaFile(item, path, remoteProjectLocale, remoteProjectId)];
                    default:
                        return [2 /*return*/, item];
                }
                return [2 /*return*/];
            });
        });
    };
    // reassigned locale to avoid wrong preview ids
    // each item has its own locale
    CaaSMapper.prototype.setLocaleFromCaasItem = function (item) {
        if (item.locale) {
            this.locale = item.locale.language + '_' + item.locale.country;
        }
    };
    CaaSMapper.prototype.mapFilterResponse = function (unmappedItems, additionalParams, filterContext, remoteProjectLocale, remoteProjectId) {
        return __awaiter(this, void 0, void 0, function () {
            var items, _a, mappedItems, remoteReferencesValues, remoteReferencesMerged;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!(additionalParams === null || additionalParams === void 0 ? void 0 : additionalParams.keys)) return [3 /*break*/, 1];
                        _a = unmappedItems; // don't map data, if additional params have been set
                        return [3 /*break*/, 3];
                    case 1: return [4 /*yield*/, Promise.all(unmappedItems.map(function (unmappedItem, index) {
                            _this.setLocaleFromCaasItem(unmappedItem);
                            switch (unmappedItem.fsType) {
                                case 'Dataset':
                                    return _this.mapDataset(unmappedItem, [(0, MappingUtils_1.getItemId)(unmappedItem, remoteProjectId)], remoteProjectLocale, remoteProjectId);
                                case 'PageRef':
                                    return _this.mapPageRef(unmappedItem, [(0, MappingUtils_1.getItemId)(unmappedItem, remoteProjectId)], remoteProjectLocale, remoteProjectId);
                                case 'Media':
                                    return _this.mapMedia(unmappedItem, [(0, MappingUtils_1.getItemId)(unmappedItem, remoteProjectId)], remoteProjectLocale, remoteProjectId);
                                case 'GCAPage':
                                    return _this.mapGCAPage(unmappedItem, [(0, MappingUtils_1.getItemId)(unmappedItem, remoteProjectId)], remoteProjectLocale, remoteProjectId);
                                case 'ProjectProperties':
                                    return _this.mapProjectProperties(unmappedItem, [(0, MappingUtils_1.getItemId)(unmappedItem, remoteProjectId)], remoteProjectLocale, remoteProjectId);
                                default:
                                    _this.logger.warn("Item at index [" + index + "] with type [" + unmappedItem.fsType + "] could not be mapped!");
                                    return unmappedItem;
                            }
                        }))];
                    case 2:
                        _a = (_b.sent()).filter(Boolean);
                        _b.label = 3;
                    case 3:
                        items = _a;
                        // cache items to avoid fetching them multiple times
                        items.forEach(function (item) {
                            if (item) {
                                _this.addToResolvedReferences(item, remoteProjectId);
                            }
                        });
                        if (!(items.length > 0)) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.resolveAllReferences(filterContext)
                            // find resolved refs and puzzle them back together
                        ];
                    case 4:
                        _b.sent();
                        _b.label = 5;
                    case 5:
                        mappedItems = (0, MappingUtils_1.findResolvedReferencesByIds)(items.map(function (item) { return (0, MappingUtils_1.getItemId)(item, remoteProjectId); }), this.resolvedReferences);
                        remoteReferencesValues = Object.values(this._remoteReferences);
                        remoteReferencesMerged = remoteReferencesValues.length > 0
                            ? remoteReferencesValues.reduce(function (result, current) {
                                return Object.assign(result, current);
                            })
                            : {};
                        // return
                        return [2 /*return*/, {
                                mappedItems: mappedItems,
                                referenceMap: __assign(__assign({}, this._referencedItems), remoteReferencesMerged),
                                resolvedReferences: this.resolvedReferences,
                            }];
                }
            });
        });
    };
    /**
     * Calls ResolveReferences for currentProject and each RemoteProject
     *
     * @param data
     * @returns data
     */
    CaaSMapper.prototype.resolveAllReferences = function (filterContext) {
        return __awaiter(this, void 0, void 0, function () {
            var remoteIds;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.referenceDepth >= this.maxReferenceDepth) {
                            // hint: handle unresolved references TNG-1169
                            this.logger.warn("Maximum reference depth of " + this.maxReferenceDepth + " has been exceeded.");
                            return [2 /*return*/];
                        }
                        this.referenceDepth++;
                        remoteIds = Object.keys(this._remoteReferences);
                        this.logger.debug('CaaSMapper.resolveAllReferences', { remoteIds: remoteIds });
                        return [4 /*yield*/, Promise.all(__spreadArray([
                                this.resolveReferencesPerProject(undefined, filterContext)
                            ], remoteIds.map(function (remoteId) {
                                return _this.resolveReferencesPerProject(remoteId, filterContext);
                            }), true))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * This method will create a filter for all referenced items that are registered inside of the referencedItems
     * and fetch them in a single CaaS-Request. If remoteProjectId is set, referenced Items from the remoteProject are fetched
     * After a successful fetch all references in the json structure will be replaced with the fetched and mapped item
     */
    CaaSMapper.prototype.resolveReferencesPerProject = function (remoteProjectId, filterContext) {
        return __awaiter(this, void 0, void 0, function () {
            var referencedItems, remoteProjectData, remoteProjectLocale, locale, referencedIds, resolvedIdsArray, resolvedIds, idsToFetchFromCaaS, idChunks;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.logger.debug('CaaSMapper.resolveReferencesPerProject', {
                            remoteProjectId: remoteProjectId,
                        });
                        referencedItems = remoteProjectId
                            ? this._remoteReferences[remoteProjectId]
                            : this._referencedItems;
                        remoteProjectData = this.getRemoteConfigForProject(remoteProjectId);
                        remoteProjectLocale = remoteProjectData === null || remoteProjectData === void 0 ? void 0 : remoteProjectData.locale;
                        locale = remoteProjectLocale || this.locale;
                        referencedIds = Object.keys(referencedItems);
                        resolvedIdsArray = Object.keys(this.resolvedReferences);
                        resolvedIds = new Set(resolvedIdsArray);
                        idsToFetchFromCaaS = referencedIds
                            .filter(function (id) { return !resolvedIds.has(id) && !_this._processedItems[id]; })
                            .map(function (id) { return id.substring(id.indexOf('#') + 1, id.indexOf('.')); });
                        referencedIds.forEach(function (id) { return (_this._processedItems[id] = true); });
                        idChunks = (0, lodash_1.chunk)(idsToFetchFromCaaS, REFERENCED_ITEMS_CHUNK_SIZE);
                        // skip stringification if logLevel is higher than debug
                        this.logger.logLevel === Logger_1.LogLevel.DEBUG &&
                            this.logger.debug('CaaSMapper.resolveReferencesPerProject: Id data', {
                                project: remoteProjectId || 'localProject',
                                resolvedIdsArray: resolvedIdsArray,
                                referencedIds: referencedIds,
                                idsToFetchFromCaaS: idsToFetchFromCaaS,
                            });
                        if (!(idsToFetchFromCaaS.length > 0)) return [3 /*break*/, 2];
                        // we pass "this" as context to fetchByFilter
                        return [4 /*yield*/, Promise.all(idChunks.map(function (ids) {
                                return _this.api.fetchByFilter({
                                    filters: [
                                        {
                                            operator: _1.ComparisonQueryOperatorEnum.IN,
                                            value: ids,
                                            field: 'identifier',
                                        },
                                    ],
                                    locale: remoteProjectLocale || locale,
                                    pagesize: REFERENCED_ITEMS_CHUNK_SIZE,
                                    remoteProject: remoteProjectId,
                                    filterContext: filterContext,
                                    normalized: true,
                                }, _this);
                            }))];
                    case 1:
                        // we pass "this" as context to fetchByFilter
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        this.logger.debug('CaaSMapper.resolveReferencesPerProject: Nothing to fetch');
                        _a.label = 3;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    CaaSMapper.prototype.getRemoteConfigForProject = function (projectId) {
        return projectId
            ? Object.values(this.api.remotes || {}).find(function (entry) { return entry.id === projectId; })
            : undefined;
    };
    return CaaSMapper;
}());
exports.CaaSMapper = CaaSMapper;
//# sourceMappingURL=CaaSMapper.js.map