"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FSXAProxyApi = void 0;
var enums_1 = require("../enums");
var Logger_1 = require("./Logger");
var MappingUtils_1 = require("./MappingUtils");
var exceptions_1 = require("../exceptions");
/**
 * This class represents the functionality of the FSXA-API as a proxy.
 * To use this class, there must be an instance of FSXARemoteApi that is accessed here.
 * By using this class, no secrets are provided to the client.
 * @see FSXARemoteApi
 */
var FSXAProxyApi = /** @class */ (function () {
    /**
     * Creates a new instance with the connection to the FSXARemoteAPI
     * @param baseUrl specifies the URL to communicate with
     * @param logLevel specifies the restrictions of logs which will be displayed
     * @param filterOptions optional {@link ProxyApiFilterOptions ProxyApiFilterOptions} (EXPERIMENTAL)
     */
    function FSXAProxyApi(baseUrl, logLevel, filterOptions) {
        if (logLevel === void 0) { logLevel = Logger_1.LogLevel.ERROR; }
        this.mode = 'proxy';
        this._baseUrl = this.baseUrl;
        this._method = 'POST';
        this._headers = {
            'Content-Type': 'application/json',
        };
        this._enableEventStream = false;
        this.baseUrl = baseUrl;
        this._logger = new Logger_1.Logger(logLevel, 'FSXAProxyApi');
        this._filterContextProvider = filterOptions === null || filterOptions === void 0 ? void 0 : filterOptions.filterContextProvider;
        this._logger.debug('FSXAProxyApi created', { baseUrl: baseUrl });
    }
    Object.defineProperty(FSXAProxyApi.prototype, "baseUrl", {
        /**
         * This method requests the current state of baseUrl
         * @param baseUrl specifies the base Url for the FSXA-Proxy-Api
         */
        get: function () {
            return this._baseUrl;
        },
        /**
         * This method sets a new state of baseUrl
         * @param baseUrl specifies the base Url for the FSXA-Proxy-Api
         */
        set: function (value) {
            value = value.trim();
            if (value === '') {
                throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.MISSING_BASE_URL, enums_1.HttpStatus.NOT_FOUND);
            }
            this._baseUrl = value;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * This methods fetches a specific element by its Id and its fetch options
     * @param id specifies the element that needs to be fetched
     * @param locale specifies the language e.g. de_DE
     * @param additionalParams sets parameters for the fetching process
     * @param remoteProject specifies the remote Project
     * @param fetchOptions specifies the options for the fetching process
     * @returns a JSON from the fetched element
     */
    FSXAProxyApi.prototype.fetchElement = function (_a) {
        var id = _a.id, locale = _a.locale, _b = _a.additionalParams, additionalParams = _b === void 0 ? {} : _b, remoteProject = _a.remoteProject, fetchOptions = _a.fetchOptions;
        return __awaiter(this, void 0, void 0, function () {
            var filterContext, body, response, _c, bodyString, errorMessage, jsonRes, _d, mappedItems, referenceMap, resolvedReferences;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        filterContext = this._filterContextProvider
                            ? this._filterContextProvider()
                            : undefined;
                        body = {
                            id: id,
                            locale: locale,
                            additionalParams: additionalParams,
                            remote: remoteProject,
                            filterContext: filterContext,
                            normalized: true,
                        };
                        this._logger.debug('fetchElement', 'trying to fetch body', body);
                        return [4 /*yield*/, this.fetch({
                                url: enums_1.FSXAProxyRoutes.FETCH_ELEMENT_ROUTE,
                                options: __assign({ method: this._method, headers: this._headers, body: body }, fetchOptions),
                            })];
                    case 1:
                        response = _e.sent();
                        if (!!response.ok) return [3 /*break*/, 6];
                        _c = response.status;
                        switch (_c) {
                            case enums_1.HttpStatus.NOT_FOUND: return [3 /*break*/, 2];
                            case enums_1.HttpStatus.UNAUTHORIZED: return [3 /*break*/, 3];
                        }
                        return [3 /*break*/, 4];
                    case 2: throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.NOT_FOUND, enums_1.HttpStatus.NOT_FOUND);
                    case 3: throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.NOT_AUTHORIZED, enums_1.HttpStatus.UNAUTHORIZED);
                    case 4: return [4 /*yield*/, response.text()];
                    case 5:
                        bodyString = _e.sent();
                        errorMessage = enums_1.FSXAApiErrors.UNKNOWN_ERROR +
                            ' Response: ' +
                            response.status +
                            ' ' +
                            bodyString;
                        throw new exceptions_1.HttpError(errorMessage, enums_1.HttpStatus.BAD_REQUEST);
                    case 6: return [4 /*yield*/, response.json()];
                    case 7:
                        jsonRes = _e.sent();
                        _d = jsonRes, mappedItems = _d.mappedItems, referenceMap = _d.referenceMap, resolvedReferences = _d.resolvedReferences;
                        mappedItems = (0, MappingUtils_1.denormalizeResolvedReferences)(mappedItems, referenceMap, resolvedReferences, remoteProject);
                        return [2 /*return*/, mappedItems[0]];
                }
            });
        });
    };
    /**
     * This method fetches elements found by a specified Filter
     * @param filters specifies the filter that needs to be fetched
     * @param locale specifies the language
     * @param page specifies the page
     * @param pagesize specifies the number of pages
     * @param additionalParams specifies additional parameters for the fetching process
     * @param remoteProject specifies the remote project for the fetching process
     * @param fetchOptions specifies options in the fetching process
     * @param sort specifies the fields to sort. Default is by id descending. Multiple sortParams are possible. First parameter is prioritized over subsequent.
     * @returns a JSON from the fetched elements
     *
     * @example
     * ```
     * const response = await api.fetchByFilter({filters: [], locale: de'DE', page: 1, pagesize: 10, additionalParams: {}, remoteProject: 'my-project'})
     * ```
     */
    FSXAProxyApi.prototype.fetchByFilter = function (_a) {
        var filters = _a.filters, locale = _a.locale, _b = _a.page, page = _b === void 0 ? 1 : _b, _c = _a.pagesize, pagesize = _c === void 0 ? 30 : _c, _d = _a.additionalParams, additionalParams = _d === void 0 ? {} : _d, remoteProject = _a.remoteProject, fetchOptions = _a.fetchOptions, _e = _a.sort, sort = _e === void 0 ? [] : _e;
        return __awaiter(this, void 0, void 0, function () {
            var filterContext, body, response, jsonRes, referenceMap, items, resolvedReferences, totalPages, size;
            return __generator(this, function (_f) {
                switch (_f.label) {
                    case 0:
                        if (pagesize < 1) {
                            this._logger.warn('fetchByFilter', 'pagesize must be greater than zero! Using fallback of 30.');
                            pagesize = 30;
                        }
                        if (page < 1) {
                            this._logger.warn('fetchByFilter', 'page must be greater than zero! Using fallback of 1.');
                            page = 1;
                        }
                        filterContext = this._filterContextProvider
                            ? this._filterContextProvider()
                            : undefined;
                        body = {
                            filters: filters,
                            locale: locale,
                            page: page,
                            pagesize: pagesize,
                            sort: sort,
                            additionalParams: __assign(__assign({}, additionalParams), { rep: 'hal' }),
                            remote: remoteProject,
                            filterContext: filterContext,
                            normalized: true,
                        };
                        this._logger.debug('fetchByFilter', 'trying to fetch with body', body);
                        return [4 /*yield*/, this.fetch({
                                url: enums_1.FSXAProxyRoutes.FETCH_BY_FILTER_ROUTE,
                                options: __assign({ method: this._method, headers: this._headers, body: body }, fetchOptions),
                            })];
                    case 1:
                        response = _f.sent();
                        this._logger.info('fetchByFilter', 'response', {
                            url: response.url,
                            status: response.status,
                        });
                        if (!response.ok) {
                            switch (response.status) {
                                case 401:
                                    throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.NOT_AUTHORIZED, enums_1.HttpStatus.UNAUTHORIZED);
                                default:
                                    throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.UNKNOWN_ERROR, enums_1.HttpStatus.BAD_REQUEST);
                            }
                        }
                        return [4 /*yield*/, response.json()];
                    case 2:
                        jsonRes = (_f.sent());
                        referenceMap = jsonRes.referenceMap, items = jsonRes.items, resolvedReferences = jsonRes.resolvedReferences, totalPages = jsonRes.totalPages, size = jsonRes.size;
                        items = (0, MappingUtils_1.denormalizeResolvedReferences)(items, referenceMap, resolvedReferences, remoteProject);
                        return [2 /*return*/, { page: page, pagesize: pagesize, totalPages: totalPages, size: size, items: items }];
                }
            });
        });
    };
    /**
     * This method fetches the navigation
     * @param initialPath to identify the navigation
     * @param locale specifies the Language
     * @param fetchOptions specifies options in the fetching Process
     * @returns a JSON from the fetching process
     */
    FSXAProxyApi.prototype.fetchNavigation = function (_a) {
        var initialPath = _a.initialPath, locale = _a.locale, fetchOptions = _a.fetchOptions;
        return __awaiter(this, void 0, void 0, function () {
            var filterContext, body, response;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        this._logger.debug('fetchNavigation', 'start', {
                            locale: locale,
                            initialPath: initialPath,
                        });
                        filterContext = this._filterContextProvider
                            ? this._filterContextProvider()
                            : undefined;
                        body = {
                            initialPath: initialPath,
                            locale: locale,
                            filterContext: filterContext,
                        };
                        this._logger.debug('fetchNavigation', 'body', { body: body });
                        return [4 /*yield*/, this.fetch({
                                url: enums_1.FSXAProxyRoutes.FETCH_NAVIGATION_ROUTE,
                                options: __assign({ method: this._method, headers: this._headers, body: body }, fetchOptions),
                            })];
                    case 1:
                        response = _b.sent();
                        this._logger.debug('fetchNavigation', 'response', {
                            url: response.url,
                            status: response.status,
                        });
                        if (!response.ok) {
                            switch (response.status) {
                                case enums_1.HttpStatus.NOT_FOUND:
                                    throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.NOT_FOUND, enums_1.HttpStatus.NOT_FOUND);
                                default:
                                    throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.UNKNOWN_ERROR, enums_1.HttpStatus.BAD_REQUEST);
                            }
                        }
                        return [2 /*return*/, response.json()];
                }
            });
        });
    };
    /**
     * This method fetches Project Properties
     * @param locale specifies the language
     * @param resolver
     * @param fetchOptions specifies options in the fetching process
     * @returns a JSON from the fetching results
     */
    FSXAProxyApi.prototype.fetchProjectProperties = function (_a) {
        var locale = _a.locale, _b = _a.resolver, resolver = _b === void 0 ? ['GCAPage'] : _b, fetchOptions = _a.fetchOptions;
        return __awaiter(this, void 0, void 0, function () {
            var filterContext, body, response, data, denormalizedProjectPropertiesArray, resolveElements, denormalizedProjectProperties;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        this._logger.debug('fetchProjectProperties', 'start', { locale: locale });
                        filterContext = this._filterContextProvider
                            ? this._filterContextProvider()
                            : undefined;
                        body = {
                            locale: locale,
                            resolver: resolver,
                            filterContext: filterContext,
                            normalized: true,
                        };
                        this._logger.debug('fetchProjectProperties', 'trying to fetch body', body);
                        return [4 /*yield*/, this.fetch({
                                url: enums_1.FSXAProxyRoutes.FETCH_PROPERTIES_ROUTE,
                                options: __assign({ method: this._method, headers: this._headers, body: body }, fetchOptions),
                            })];
                    case 1:
                        response = _c.sent();
                        this._logger.debug('fetchProjectProperties', 'response', {
                            url: response.url,
                            status: response.status,
                        });
                        if (!response.ok) {
                            switch (response.status) {
                                case 404:
                                    throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.NOT_FOUND, enums_1.HttpStatus.NOT_FOUND);
                                default:
                                    throw new exceptions_1.HttpError(enums_1.FSXAApiErrors.UNKNOWN_ERROR, enums_1.HttpStatus.BAD_REQUEST);
                            }
                        }
                        return [4 /*yield*/, response.json()];
                    case 2:
                        data = (_c.sent());
                        if (!data) {
                            this._logger.debug('fetchProjectProperties', 'no data found');
                            return [2 /*return*/, null];
                        }
                        denormalizedProjectPropertiesArray = (0, MappingUtils_1.denormalizeResolvedReferences)([data.projectProperties], data.projectPropertiesReferenceMap || {}, data.projectPropertiesResolvedReferences || {});
                        resolveElements = (0, MappingUtils_1.denormalizeResolvedReferences)(data.resolveItems, data.resolveReferenceMap || {}, data.resolveResolvedReferences || {});
                        denormalizedProjectProperties = denormalizedProjectPropertiesArray.length > 0
                            ? denormalizedProjectPropertiesArray[0]
                            : null;
                        if (!denormalizedProjectProperties)
                            return [2 /*return*/, null
                                //Insert fetched Data into projectProperties
                            ];
                        //Insert fetched Data into projectProperties
                        resolveElements.forEach(function (element) {
                            denormalizedProjectProperties.data[data.idToKeyMap[element.id]] =
                                element.data;
                        });
                        return [2 /*return*/, denormalizedProjectProperties];
                }
            });
        });
    };
    /**
     * Getter/Setter to enable the CaaS event stream
     * @returns true, if a event stream should pipe events from CaaS change events websocket
     */
    FSXAProxyApi.prototype.enableEventStream = function (enable) {
        if (typeof enable !== 'undefined')
            this._enableEventStream = enable;
        return this._enableEventStream;
    };
    /**
     * This method initilises an `EventSoure` pointing to the CaaS change event stream.
     * @param additionalParams sets parameters for the fetching process
     * @param remoteProject specifies the remote Project
     * @returns an EventSource
     */
    FSXAProxyApi.prototype.connectEventStream = function (_a) {
        var _b = _a === void 0 ? {} : _a, remoteProject = _b.remoteProject;
        var url = new URL(this.baseUrl + enums_1.FSXAProxyRoutes.STREAM_CHANGE_EVENTS_ROUTE, location.origin);
        if (remoteProject) {
            url.searchParams.set('remoteProject', remoteProject);
        }
        this._logger.info('connectEventStream', 'start', "" + url);
        return new EventSource(url.toString());
    };
    FSXAProxyApi.prototype.fetch = function (_a) {
        var _b, _c;
        var url = _a.url, options = _a.options;
        if ((options === null || options === void 0 ? void 0 : options.body) && typeof options.body === 'object') {
            options.body = JSON.stringify(options.body);
        }
        this._logger.debug('fetch', 'start', {
            baseUrl: this.baseUrl,
            url: url,
            options: options,
            isServer: typeof process === 'undefined' ? undefined : (_b = process) === null || _b === void 0 ? void 0 : _b.server,
            isClient: typeof process === 'undefined' ? undefined : (_c = process) === null || _c === void 0 ? void 0 : _c.client,
            window: typeof window,
        });
        return fetch(this.baseUrl + url, options);
    };
    return FSXAProxyApi;
}());
exports.FSXAProxyApi = FSXAProxyApi;
//# sourceMappingURL=FSXAProxyApi.js.map