"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = exports.LogLevel = void 0;
var util_1 = require("util");
var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["DEBUG"] = 0] = "DEBUG";
    LogLevel[LogLevel["INFO"] = 1] = "INFO";
    LogLevel[LogLevel["WARNING"] = 2] = "WARNING";
    LogLevel[LogLevel["ERROR"] = 3] = "ERROR";
    LogLevel[LogLevel["NONE"] = 4] = "NONE";
})(LogLevel = exports.LogLevel || (exports.LogLevel = {}));
var getCircularReplacer = function () {
    var seen = new WeakMap();
    return function (key, value) {
        if (typeof value === 'object' && value !== null) {
            if (seen.has(value)) {
                var representation = [];
                var firstSeen = seen.get(value);
                firstSeen && representation.push("first occurence: " + firstSeen);
                value.type && representation.push("type: " + value.type);
                value.id && representation.push("id: " + value.id);
                return "[~circle. " + representation.join(', ') + "]";
            }
            seen.set(value, key);
        }
        return value;
    };
};
var formatOutput = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    args = args.map(function (entry) {
        if (typeof entry === 'object')
            return JSON.stringify(entry, getCircularReplacer());
        return entry;
    });
    return (0, util_1.inspect)(args.join(' | '), {
        showHidden: false,
        depth: null,
        colors: false,
        compact: true,
        breakLength: Infinity,
    }).replace(/\'/g, '');
};
var identity = function (text) { return text; };
var Logger = /** @class */ (function () {
    function Logger(logLevel, name, chalk) {
        this._logLevel = logLevel;
        this._name = name;
        this.chalk = chalk;
    }
    Object.defineProperty(Logger.prototype, "logLevel", {
        get: function () {
            return this._logLevel;
        },
        enumerable: false,
        configurable: true
    });
    Logger.prototype.debug = function () {
        var _a, _b;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (this._logLevel <= LogLevel.DEBUG) {
            console.info((((_a = this.chalk) === null || _a === void 0 ? void 0 : _a.gray) || identity)((((_b = this.chalk) === null || _b === void 0 ? void 0 : _b.bgWhite.black) || identity)(' DEBUG ') + " " + this._name + " | " + formatOutput.apply(void 0, args)));
        }
    };
    Logger.prototype.log = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this.info(args);
    };
    Logger.prototype.info = function () {
        var _a, _b;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (this._logLevel <= LogLevel.INFO) {
            console.info((((_a = this.chalk) === null || _a === void 0 ? void 0 : _a.blue) || identity)((((_b = this.chalk) === null || _b === void 0 ? void 0 : _b.bgBlue.white) || identity)(' INFO ') + " " + this._name + " | " + formatOutput.apply(void 0, args)));
        }
    };
    Logger.prototype.warn = function () {
        var _a, _b;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (this._logLevel <= LogLevel.WARNING) {
            console.warn((((_a = this.chalk) === null || _a === void 0 ? void 0 : _a.yellow) || identity)((((_b = this.chalk) === null || _b === void 0 ? void 0 : _b.bgYellow.black) || identity)(' WARN ') + " " + this._name + " | " + formatOutput.apply(void 0, args)));
        }
    };
    Logger.prototype.error = function () {
        var _a, _b;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (this._logLevel <= LogLevel.ERROR) {
            console.error((((_a = this.chalk) === null || _a === void 0 ? void 0 : _a.red) || identity)((((_b = this.chalk) === null || _b === void 0 ? void 0 : _b.bgRed.black) || identity)(' ERROR ') + " " + this._name + " | " + formatOutput.apply(void 0, args)));
        }
    };
    return Logger;
}());
exports.Logger = Logger;
//# sourceMappingURL=Logger.js.map