"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.denormalizeResolvedReferences = exports.findResolvedReferencesByIds = exports.getItemId = void 0;
var lodash_1 = require("lodash");
var IMAGE_MAP_PLACEHOLDER = 'IMAGEMAP';
var IMAGE_MAP_RESOLUTION_SPLIT_DELIMITER = '___';
var getItemId = function (item, remoteProjectId) {
    var _a;
    var projectPrefix = remoteProjectId ? remoteProjectId + "#" : '';
    if (item.fsType === 'ProjectProperties') {
        var caasApiItem = item;
        var locale = caasApiItem._id.split('.')[1];
        return "" + projectPrefix + item.identifier + "." + locale;
    }
    return "" + projectPrefix + (item.previewId || ((_a = item) === null || _a === void 0 ? void 0 : _a._id));
};
exports.getItemId = getItemId;
// we use a query function, because ids get mixed up:
// referencedItems store identifier, resolvedReferences store _id or previewId
var findResolvedReferencesByIds = function (ids, resolvedReferences) {
    return ids.map(function (id) { return resolvedReferences[id]; }).filter(function (item) { return item; });
};
exports.findResolvedReferencesByIds = findResolvedReferencesByIds;
// Check for image map and force to single resolution
var imageMapForceResolution = function (_a) {
    var _b;
    var resolvedReferences = _a.resolvedReferences, path = _a.path, referencedId = _a.referencedId;
    // get the registered reference item value by path
    var registeredReferenceItem = (0, lodash_1.get)(resolvedReferences, path);
    // check if the registered reference item is an ImageMap
    if (!registeredReferenceItem ||
        typeof registeredReferenceItem !== 'string' ||
        !registeredReferenceItem.startsWith(IMAGE_MAP_PLACEHOLDER)) {
        return null;
    }
    // Get the forced resolution from the registered referenced item image map
    // e.g. IMAGEMAP___{RESOLUTION}___{IMAGE_ID}
    var segments = registeredReferenceItem.split(IMAGE_MAP_RESOLUTION_SPLIT_DELIMITER);
    // Check if it is exactly 3 parts
    if (!segments || segments.length !== 3) {
        return null;
    }
    var forcedResolution = segments[1];
    // get the Image Object from the resolved references by referencedId -> ImageMapId
    var resolvedImage = __assign({}, resolvedReferences[referencedId]);
    // force to a single resolution
    resolvedImage.resolutions = (_b = {},
        _b[forcedResolution] = resolvedImage.resolutions[forcedResolution],
        _b);
    return resolvedImage;
};
var denormalizeResolvedReferences = function (mappedItems, referenceMap, resolvedReferences, remoteProjectId) {
    if (!referenceMap || Object.keys(referenceMap).length === 0)
        return mappedItems;
    var _loop_1 = function (referencedId, occurences) {
        // Iterate over all insertion paths and insert references into objects
        occurences.forEach(function (path) {
            if (resolvedReferences[referencedId]) {
                var resolvedImage = imageMapForceResolution({
                    resolvedReferences: resolvedReferences,
                    path: path,
                    referencedId: referencedId,
                });
                (0, lodash_1.set)(resolvedReferences, path, resolvedImage || resolvedReferences[referencedId]);
            }
            else {
                console.warn("[denormalizeResolvedReferences] Unable to find object [" + referencedId + "] during denormalization. resolvedReferencesKeys: {[" + Object.keys(resolvedReferences).join('],[') + "]}");
            }
        });
    };
    // denormalize
    for (var _i = 0, _a = Object.entries(referenceMap); _i < _a.length; _i++) {
        var _b = _a[_i], referencedId = _b[0], occurences = _b[1];
        _loop_1(referencedId, occurences);
    }
    // update mappedItems
    var queriedIds = mappedItems
        .filter(function (item) { return !!item; })
        .map(function (item) { return getItemId(item, remoteProjectId); });
    return findResolvedReferencesByIds(queriedIds, resolvedReferences);
};
exports.denormalizeResolvedReferences = denormalizeResolvedReferences;
//# sourceMappingURL=MappingUtils.js.map