"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.replaceUnnestedLinkTemplatesWithOneLinkElement = exports.isValidRegex = exports.removeFromIdMap = exports.removeFromStructure = exports.removeFromSeoRouteMap = void 0;
var removeFromSeoRouteMap = function (seoRouteMap, ids) {
    Object.keys(seoRouteMap).forEach(function (key) {
        if (!ids.includes(seoRouteMap[key])) {
            delete seoRouteMap[key];
        }
    });
    return seoRouteMap;
};
exports.removeFromSeoRouteMap = removeFromSeoRouteMap;
var removeFromStructure = function (structure, ids) {
    return structure.filter(function (item) {
        if (ids.includes(item.id)) {
            item.children = (0, exports.removeFromStructure)(item.children, ids);
            return true;
        }
        return false;
    });
};
exports.removeFromStructure = removeFromStructure;
var removeFromIdMap = function (idMap, ids) {
    var copyOfIdMap = __assign({}, idMap);
    for (var route in copyOfIdMap) {
        if (!ids.includes(route)) {
            delete copyOfIdMap[route];
        }
    }
    return copyOfIdMap;
};
exports.removeFromIdMap = removeFromIdMap;
var isValidRegex = function (str) {
    try {
        // throws an error if not a valid regex
        var regex = new RegExp(str);
        return true;
    }
    catch (e) {
        return false;
    }
};
exports.isValidRegex = isValidRegex;
function findOpeningLinkTags(input) {
    var openingTagRegex = /<div data-fs-type=\\*?"link\.(.*?)\\*?">\\*?<script type=\\*?"application\/json\\*?">/;
    var result = [];
    var i = 0;
    while (i < input.length) {
        var openingTagMatch = openingTagRegex.exec(input.substring(i));
        if (openingTagMatch) {
            result.push({
                index: openingTagMatch.index + i,
                openingTag: openingTagMatch[0],
            });
            i += openingTagMatch.index + openingTagMatch[0].length;
        }
        else {
            break;
        }
    }
    return result;
}
// restructure the link structure into one single link element  (hint: *? matches non-eager)
// we need to find only the top-level link templates and replace them
// as there can be link templates inside link templates as well as multiple top-level link templates in one element
// the tricky part is that the closing tag for a nested link template does not "end" the wrapping link template
function replaceUnnestedLinkTemplatesWithOneLinkElement(input, ENTITIES) {
    var _a, _b;
    var openingTags = findOpeningLinkTags(input);
    // replace link templates if there are any
    if (openingTags.length <= 0) {
        return input;
    }
    var openingTagRegex = /<div data-fs-type=\\*?"link\.(.*?)\\*?">\\*?<script type=\\*?"application\/json\\*?">/;
    var closingTagRegex = /<\/script>\s*<a>(.*?)<\/a>\s*<\/div>/;
    var result = input;
    var openingTagsIndex = 0;
    var currentIndex = openingTags[openingTagsIndex].index;
    var stack = [];
    stack.push(openingTags[openingTagsIndex]);
    while (stack.length > 0 && currentIndex < input.length) {
        // Find the next closing tag between opening index and next opening tag (or input end if there are no more opening tags)
        var nextOpeningTag = ((_a = openingTags[openingTagsIndex + 1]) === null || _a === void 0 ? void 0 : _a.index) || input.length;
        var closingTags = closingTagRegex.exec(input.substring(currentIndex, nextOpeningTag));
        if (closingTags) {
            // if we are at outermost level (not nested into another link template) replace the element in result with link element
            if (stack.length === 1) {
                var realClosingTagIndex = currentIndex + closingTags.index;
                var opening = stack[0];
                var elementToReplace = input.substring(opening.index, realClosingTagIndex + closingTags[0].length);
                var data = input
                    .substring(opening.index + opening.openingTag.length, realClosingTagIndex)
                    // Since java 4 backslashes are considered as one backslash, we need to replace them with two backslashes
                    .replace(/\\\\/g, '\\')
                    .replace(/(["&<\\])/g, function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    return ENTITIES.get(args[1]) || args[1];
                })
                    .replace(/&#34;/g, '\\&quot;');
                var type = opening.openingTag.replace(openingTagRegex, function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    return args[1];
                });
                var content = closingTags[0].replace(closingTagRegex, function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    return args[1];
                });
                result = result.replace(elementToReplace, "<link type=\"" + type + "\" data=\"" + data + "\">" + content + "</link>");
            }
            // remove the corresponding opening tag from stack as it is closed now
            stack.pop();
            // set current index to end of closing tag, this is where we want to continue our search for closing tags
            currentIndex += closingTags.index + closingTags[0].length;
        }
        else {
            // no closing tag was found, set the current index to the next opening tag or to the end of the input if there are no more opening tags
            currentIndex = ((_b = openingTags[openingTagsIndex + 1]) === null || _b === void 0 ? void 0 : _b.index) || input.length;
        }
        // add the next opening tag to the stack if there are any
        // but only if there are no open tags left, or there are open tags but the next opening tag needs to be checked as there was no closing tag found in the currently checked substring
        if (openingTagsIndex < openingTags.length - 1 &&
            (stack.length == 0 ||
                openingTags[openingTagsIndex + 1].index <= currentIndex)) {
            openingTagsIndex += 1;
            stack.push(openingTags[openingTagsIndex]);
        }
    }
    return result;
}
exports.replaceUnnestedLinkTemplatesWithOneLinkElement = replaceUnnestedLinkTemplatesWithOneLinkElement;
//# sourceMappingURL=utils.js.map