declare type AvailableLocaleParams = {
    navigationServiceURL: string;
    projectId: string;
    contentMode: string | ('preview' | 'release');
};
/**
 * This method fetches and aggregate available locale from navigation service.
 * The {@link AvailableLocaleParams AvailableLocaleParams} object defines options to specify your request.
 * Example call:
 *
 * ```typescript
  const availableLocales = await getAvailableLocales({
    navigationServiceURL: 'https://your.navigation-service.url/navigation',
    projectId: 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx',
    contentMode: 'preview',
  })
 * ```
 * @param navigationServiceURL the url of navigation service
 * @param projectId the project id to use
 * @param contentMode the content mode to use, either 'preview' or 'release'
 * @returns an array of available locales in ISO format. e.g. ['en_GB', 'de_DE']
 */
export declare const getAvailableLocales: ({ navigationServiceURL, projectId, contentMode, }: AvailableLocaleParams) => Promise<string[]>;
export {};
