import { FetchByFilterParams, FetchElementParams, FetchNavigationParams, FetchProjectPropertiesParams } from '../types';
export declare type ParamValidationResult<T extends ParamValidationErrorReason> = ParamValidationSuccess | ParamValidationError<T>;
export declare type ParamValidationSuccess = {
    valid: true;
};
/**
 * All Existing Reasons to reject param Validation
 */
export declare type ParamValidationErrorReason = 'MISSING_PARAM_OBJECT' | 'MISSING_LOCALE' | 'MISSING_ID' | 'MISSING_FILTERS';
export declare type ParamValidationError<T extends ParamValidationErrorReason = ParamValidationErrorReason> = Readonly<{
    valid: false;
    reason: T;
    statusCode: 400;
}>;
/**
 * Creates the correct ValidationError for the given reason.
 * @param reason
 * @returns
 */
export declare const createValidationError: <T extends ParamValidationErrorReason>(reason: T) => Readonly<{
    valid: false;
    reason: T;
    statusCode: 400;
}>;
/**
 * Checks for required Parameters
 * @param params
 * @returns result object indicating validity, and a reason if not valid.
 */
export declare const validateParamsFetchElement: (params: Partial<FetchElementParams> | null | undefined) => ParamValidationResult<'MISSING_ID' | 'MISSING_LOCALE' | 'MISSING_PARAM_OBJECT'>;
/**
 * Checks for required Parameters
 * @param params
 * @returns result object indicating validity, and a reason if not valid.
 */
export declare const validateParamsFetchNavigation: (params: Partial<FetchNavigationParams> | null | undefined) => ParamValidationResult<'MISSING_LOCALE' | 'MISSING_PARAM_OBJECT'>;
/**
 * Checks for required Parameters
 * @param params
 * @returns result object indicating validity, and a reason if not valid.
 */
export declare const validateParamsFetchByFilter: (params: Partial<FetchByFilterParams> | null | undefined) => ParamValidationResult<'MISSING_PARAM_OBJECT' | 'MISSING_FILTERS'>;
/**
 * Checks for required Parameters
 * @param params
 * @returns result object indicating validity, and a reason if not valid.
 */
export declare const validateParamsFetchProjectProperties: (params: Partial<FetchProjectPropertiesParams> | null | undefined) => ParamValidationResult<'MISSING_LOCALE' | 'MISSING_PARAM_OBJECT'>;
